"""Application constants."""

"""The application name. This is displayed on the window's title and in every exported file."""
APP_NAME = "FPGA Device Manager"

"""Application version. Displayed next to the name."""
APP_VERSION = "0.2.4"

"""Default setting for maximum number of devices."""
MAX_DEVICES = 255
